<?php
/**
 * ECalendarView class file.
 *
 * @author Martin Ludvik <matolud@gmail.com>
 * @copyright Copyright &copy; 2013 by Martin Ludvik
 * @license http://opensource.org/licenses/MIT MIT license
 */

Yii::import('ecalendarview.ECalendarViewDataProvider');

/**
 * The calendar view renders days using customizable view.
 */
class ECalendarView extends CWidget {

  /**
   * @var ECalendarViewDataProvider The data provider.
   */
  private $_dataProvider;

  /**
   * @var string The custom view that is used to render each day cell. If not provided, default view is used.
   */
  private $_itemView;

  /**
   * @var string The path to custom css file to style calendar. If not provided, default css file is used.
   */
  private $_cssFile;

  /**
   * Constructs the calendar and sets it's attributes to default values.
   * @param CBaseController $owner The owner of calendar.
   */
  public function __construct(CBaseController $owner) {
    parent::__construct($owner);
    $this->_dataProvider = new ECalendarViewDataProvider();
    $this->_itemView = null;
    $this->_cssFile = null;
  }

  /**
   * Sets the data provider's attributes.
   * @param array $config The attributes as key=>value map.
   */
  public function setDataProvider(array $config) {
    foreach($config as $key => $value) {
      $this->getDataProvider()->$key = $value;
    }
  }

  /**
   * @see ECalendarView::$_itemView
   */
  public function setItemView($itemView) {
    $this->_itemView = $itemView;
  }

  /**
   * @see ECalendarView::$_cssFile
   */
  public function setCssFile($cssFile) {
    $this->_cssFile = $cssFile;
  }

  /**
   * @see ECalendarView::$_dataProvider
   */
  public function getDataProvider() {
    return $this->_dataProvider;
  }

  /**
   * @see ECalendarView::$_itemView
   */
  public function getItemView() {
    return $this->_itemView;
  }

  /**
   * @see ECalendarView::$_cssFile
   */
  public function getCssFile() {
    return $this->_cssFile;
  }

  /**
   * Generates url for page navigation.
   * @param int $pageIndex The index of page.
   * @return string The url.
   */
  public function getUrl($pageIndex) {
    $params = $_GET;
    $params[$this->getDataProvider()->getPagination()->getPageIndexVar()] = $pageIndex;
    return Yii::app()->getController()->createUrl('', $params);
  }

  /**
   * Initializes the calendar.
   */
  public function init() {
    $cssFilePath = $this->resolveCssFilePath($this->getCssFile(), 'styles.css');
    $publishedCssFilePath = Yii::app()->getAssetManager()->publish($cssFilePath);
    Yii::app()->clientScript->registerCssFile($publishedCssFilePath);
    parent::init();
  }

  /**
   * Runs the calendar.
   */
  public function run() {
    $this->render('main', array(
      'dataProvider' => $this->getDataProvider(),
      'itemViewFile' => $this->resolveViewFile($this->getItemView(), '_view'),
    ));
  }

  /**
   * Resolves path to css file that should be used.
   * @param string $customCssFile The custom css file path.
   * @param string $defaultCssFile The default css file path.
   * @return string The css file path.
   */
  private function resolveCssFilePath($customCssFile, $defaultCssFile) {
    return $customCssFile ? $customCssFile : dirname(__FILE__) . DIRECTORY_SEPARATOR . 'assets/' . $defaultCssFile;
  }

  /**
   * Resolves the view file that should be used.
   * @param string $customView The custom view.
   * @param string $defaultView The default view.
   * @return string The view.
   */
  private function resolveViewFile($customView, $defaultView) {
    return $customView ? $this->getOwner()->getViewFile($customView) : $this->getViewFile($defaultView);
  }

}
